package br.usp.poli.java.ihc.applet;

/*
Copyright (C) 2012 Fbio Levy Siqueira (levy.siqueira@gmail.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

import javax.swing.JApplet;

import br.usp.poli.java.ihc.ErroPoquer;
import br.usp.poli.java.ihc.JogadorDePoquerFechadoIHC;
import br.usp.poli.java.ihc.JogadorDeTexasHoldemIHC;
import br.usp.poli.java.ihc.ThreadPoquer;

//Importar "plugin.jar" localizado (normalmente) em "C:\Program Files (x86)\Java\jre1.7.0\lib"
import netscape.javascript.*;

/**
 * Applet Java que comunica com a interface Web.
 * @author FLevy
 * @version 1.0
 */
@SuppressWarnings("serial")
public class AppletPoquer extends JApplet {
	private ThreadPoquer poquer;
	private InterfaceHTML interfaceHTML;
	private Configurador configurador;
	
	public void init() {
		super.init();
		try {
			this.interfaceHTML = new InterfaceHTML(JSObject.getWindow(this));
			this.configurador = new Configurador(interfaceHTML);
			this.poquer = new ThreadPoquer(this.configurador.getJogoTexasHoldem(), this.configurador.getJogoPoquerFechado());
		} catch (ErroPoquer e) {
			this.interfaceHTML.erro(e.getMessage());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Obtm a interface HTML, necessria para a comunicao com o Applet.
	 * @return O objeto InterfaceHTML.
	 */
	public InterfaceHTML getInterfaceHTML() {
		return interfaceHTML;
	}
	
	/**
	 * Obtm os nomes dos tipos de jogadores de Texas Hold'em disponveis.
	 * @return Um vetor com o nome dos tipos de jogadores de texas hold'em.
	 */
	public String[] getJogadoresDeTexasHoldem() {
		return configurador.getJogadoresDeTexasHoldem();
	}
	
	/**
	 * Obtm os nomes dos tipos de jogadores de Poquer Fechado disponveis.
	 * @return Um vetor com o nome dos tipos de jogadores de poquer fechado.
	 */
	public String[] getJogadoresDePoquerFechado() {
		return configurador.getJogadoresDePoquerFechado();
	}
	
	/**
	 * Inicia um Jogo de pquer.
	 * @param jogo O nome do jogo ("texas" ou "fechado")
	 * @param tipoAdversario O nome do tipo de aversrio.
	 * @param nomeJogador O nome do jogador.
	 */
	public void iniciar(String jogo, String tipoAdversario, String nomeJogador) {
		System.out.println("[DEBUG] Iniciando o jogo: " + jogo + ": " + tipoAdversario + " X " + nomeJogador);
		try {
			if ("texas".equals(jogo)) {
				JogadorDeTexasHoldemIHC computador = this.configurador.getComputadorDeTexasHoldem(tipoAdversario);
				JogadorDeTexasHoldemIHC usuario = this.configurador.getJogadorDeTexasHoldem(nomeJogador); 
				
				this.interfaceHTML.registrarJogadores(computador, usuario);
				this.poquer.iniciar(computador, usuario);
			} else {
				JogadorDePoquerFechadoIHC computador = this.configurador.getComputadorDePoquerFechado(tipoAdversario);
				JogadorDePoquerFechadoIHC usuario = this.configurador.getJogadorDePoquerFechado(nomeJogador); 
				
				this.interfaceHTML.registrarJogadores(computador, usuario);
				this.poquer.iniciar(computador, usuario);
			}
		} catch (ErroPoquer e) {
			e.printStackTrace();
			this.interfaceHTML.erro(e.getMessage());
		} catch (Exception e) {
			e.printStackTrace();
			this.interfaceHTML.erro(e.getLocalizedMessage());	
		}
	}
	
	public String[][] getParameterInfo() {
		return null;
	}
	
}
