package br.usp.poli.java.ihc.applet;

/*
Copyright (C) 2012 Fbio Levy Siqueira (levy.siqueira@gmail.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

import br.usp.poli.java.*;
import br.usp.poli.java.ihc.ErroPoquer;

/**
 * Classe que codifica e decodifica uma Carta, usada para comunicao 
 * com a pgina.<br />
 * A codificao usada  NAIPE-VALOR, sendo que NAIPE pode ser O, E, C e P
 * e os valores podem ser 2, 3, 4, 5, 6, 7, 8, 9, 10, J, Q, K e A.
 * @author FLevy
 */
class CartaImpl implements Carta, Comparable<Carta> {
	private int naipe;
	private int valor;
	
	private static final String[] TEXTO_NAIPES = {"O", "E", "C", "P"};
	private static final String[] TEXTO_VALORES = {"2", "3", "4", "5", "6", "7", "8", "9", "10", "J", "Q", "K", "A"};
	
	/**
	 * Cria uma carta a partir de um naipe e um valor.
	 * @param naipe O naipe (definido em Carta).
	 * @param valor O valor (definido em Carta).
	 */
	public CartaImpl(int naipe, int valor) {
		this.naipe = naipe;
		this.valor = valor;
	}
	
	/**
	 * Cria uma carta a partir de uma String.
	 * @param carta A String representando uma carta.
	 * @throws ErroPoquer
	 */
	public CartaImpl(String carta) throws ErroPoquer {
		String[] info = carta.split("\\-");
		
		if (info == null || info.length < 2) throw new ErroPoquer("Carta informada  invlida.");
		
		boolean encontrou = false;
		for (int valor = 0; valor < TEXTO_VALORES.length && !encontrou; valor++) {
			if (TEXTO_VALORES[valor].equalsIgnoreCase(info[0])) {
				encontrou = true;
				this.valor = valor;
			}
		}
		
		if (!encontrou) throw new ErroPoquer("O valor da carta informada  invlido.");
		
		encontrou = false;
		for (int naipe = 0; naipe < TEXTO_NAIPES.length && !encontrou; naipe++) {
			if (TEXTO_NAIPES[naipe].equalsIgnoreCase(info[1])) {
				encontrou = true;
				this.naipe = naipe;
			}
		}
		
		if (!encontrou) throw new ErroPoquer("O naipe da carta informada  invlido.");
	}
	
	public int getNaipe() {
		return naipe;
	}
	
	public int getValor() {
		return valor;
	}
	
	/**
	 * Obtm uma String representando a Carta em questo
	 * @param c A carta a qual se deseja uma representao em String.
	 * @return A String com a codificao usada para a carta c.
	 */
	public static String toString(Carta c) {
		return TEXTO_VALORES[c.getValor()] + "-" + TEXTO_NAIPES[c.getNaipe()];
	}
	
	public String toString() {
		return CartaImpl.toString(this);
	}

	public int compareTo(Carta c) {
		if (this.getValor() > c.getValor()) return 1;
		else if (this.getValor() < c.getValor()) return -1;
		return 0;
	}
}

