package br.usp.poli.java.ihc.applet;

/*
Copyright (C) 2012 Fbio Levy Siqueira (levy.siqueira@gmail.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

import java.util.Arrays;
import java.util.HashMap;
import java.util.Set;

import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

import br.usp.poli.java.JogadorDePoquerFechado;
import br.usp.poli.java.JogadorDeTexasHoldem;
import br.usp.poli.java.ihc.ErroPoquer;
import br.usp.poli.java.ihc.JogadorDePoquerFechadoIHC;
import br.usp.poli.java.ihc.JogadorDeTexasHoldemIHC;
import br.usp.poli.java.ihc.JogoDePoquerFechado;
import br.usp.poli.java.ihc.JogoDeTexasHoldem;

/**
 * Classe que cuida do processamento do arquivo de configurao (XML).
 * @author FLevy
 */
class Configurador {
	private static final String XML_CONFIGURACAO = "/config.xml";
	private static final String XML_JOGO = "jogo";
	private static final String XML_JOGADORES = "jogadores";
	private static final String XML_JOGADOR = "jogador";
	private static final String XML_NOME_JOGADOR = "nome";
	private static final String XML_TEXAS = "texas";
	private static final String XML_FECHADO = "fechado";
	
	private InterfaceHTML interfaceHTML;
	private HashMap<String, Class<? extends JogadorDeTexasHoldem>> jogadoresTexas;
	private HashMap<String, Class<? extends JogadorDePoquerFechado>> jogadoresFechado;
	private Class<? extends JogoDeTexasHoldem> jogoTexas;
	private Class<? extends JogoDePoquerFechado> jogoFechado;

	/**
	 * Cria um configurador.<br />
	 * Em Java  possvel obter um objeto com informaes de uma classe (usando
	 * Reflection). Esse objeto est disponvel como um atributo de classe (static)
	 * "class". Exemplo: Poquer.class retorna um objeto com as informaes da classe
	 * desta classe (Poquer).
	 * @param interfaceHTML A interface HTML usada.
	 * @throws ErroPoquer Erro, caso no existam jogadores definidos.
	 */
	public Configurador(InterfaceHTML interfaceHTML) throws ErroPoquer {
		this.interfaceHTML = interfaceHTML;
		
		XMLInputFactory factory = XMLInputFactory.newInstance();
		try {
			XMLStreamReader parser = factory.createXMLStreamReader(Configurador.class.getResourceAsStream(XML_CONFIGURACAO));
			System.out.println("[DEBUG] Obtendo configuraes no arquivo " + XML_CONFIGURACAO);
			//getClass().getClassLoader().getResourceAsStream():
			
			while (parser.hasNext()) {
				switch (parser.next()) {
				case XMLStreamConstants.START_ELEMENT:
					String tag = parser.getLocalName();
					if (XML_TEXAS.equalsIgnoreCase(tag)) {
						processarTexas(parser);
					} else if (XML_FECHADO.equalsIgnoreCase(tag)) {
						processarFechado(parser);
					}
					break;
				}
			}
			parser.close();
		} catch (XMLStreamException e1) {
			e1.printStackTrace();
			throw new ErroPoquer("Problema ao ler o arquivo '" + XML_CONFIGURACAO + "': " + e1.getLocalizedMessage());
		}
		
		if (jogoTexas == null && jogoFechado == null) {
			System.out.println("ERRO - nenhuma classe de jogo!");
			throw new ErroPoquer("No foi encontrado nenhum jogo de Poquer e jogadores disponveis.");
		}
	}
	
	/**
	 * Processa a parte do XML referente ao poquer texas.
	 * @param parser O parser atual.
	 */
	private void processarTexas(XMLStreamReader parser) throws ErroPoquer, XMLStreamException {
		String classeTexas;
		jogoTexas = null;
		jogadoresTexas = null;
		
		while(parser.hasNext()) {
			switch (parser.next()) {
			case XMLStreamConstants.START_ELEMENT:
				String tag = parser.getLocalName();
				if (XML_JOGO.equalsIgnoreCase(tag)) {
					classeTexas = parser.getElementText();
					System.out.println("[DEBUG] Classe Texas: " + classeTexas);
					if (classeTexas != null) {
						try {
							jogoTexas = Class.forName(classeTexas).asSubclass(JogoDeTexasHoldem.class);
						} catch (ClassNotFoundException e) {
							// Classe no encontrada! No est no classpath...
							throw new ErroPoquer("A classe '" + classeTexas + "' no foi encontrada no ClassPath.", e);
						} catch (ClassCastException e) {
							// Classe no  do tipo certo
							throw new ErroPoquer("A classe '" + classeTexas + "' no  um JogoDeTexasHoldem.", e);
						}
					}
				} else if (XML_JOGADORES.equalsIgnoreCase(tag)) {
					if (jogoTexas != null) {
						jogadoresTexas = obterClassesJogadoresTexas(parser);
						if (jogadoresTexas == null) {
							// Sem jogadores no d para jogar...
							jogoTexas = null;
						}
					}
				}
				break;
			case XMLStreamConstants.END_ELEMENT:
				if (XML_TEXAS.equalsIgnoreCase(parser.getLocalName())) {
					// fim
					return;
				}
			}
		}
	}
	
	/**
	 * Processa a parte do XML referente ao poquer fechado.
	 * @param parser O parser atual.
	 */
	private void processarFechado(XMLStreamReader parser) throws ErroPoquer, XMLStreamException{
		String classeFechado;
		jogoFechado = null;
		jogadoresFechado = null;
		
		while(parser.hasNext()) {
			switch (parser.next()) {
			case XMLStreamConstants.START_ELEMENT:
				String tag = parser.getLocalName();
				if (XML_JOGO.equalsIgnoreCase(tag)) {
					classeFechado = parser.getElementText();
					System.out.println("[DEBUG] Classe Poquer Fechado: " + classeFechado);
					if (classeFechado != null) {
						try {
							jogoFechado = Class.forName(classeFechado).asSubclass(JogoDePoquerFechado.class);
						} catch (ClassNotFoundException e) {
							// Classe no encontrada! No est no classpath...
							throw new ErroPoquer("A classe '" + classeFechado + "' no foi encontrada no ClassPath.", e);
						} catch (ClassCastException e) {
							// Classe no  do tipo certo
							throw new ErroPoquer("A classe '" + classeFechado + "' no  um JogoDePoquerFechado.", e);
						}
					}
				} else if (XML_JOGADORES.equalsIgnoreCase(tag)) {
					if (jogoFechado != null) {
						jogadoresFechado = obterClassesJogadoresFechado(parser);
						if (jogadoresFechado == null) {
							// Sem jogadores no d para jogar...
							jogoFechado = null;
						}
					}
				}
				break;
			case XMLStreamConstants.END_ELEMENT:
				if (XML_FECHADO.equalsIgnoreCase(parser.getLocalName())) {
					// fim
					return;
				}
			}
		}
	}
	
	/**
	 * Obtem um vetor com os nomes dos tipos de jogadores de
	 * Texas Hold'em vlidos.
	 * @return Vetor com os nomes dos tipos disponveis.
	 */
	public String[] getJogadoresDeTexasHoldem() {
		Set<String> strings = jogadoresTexas.keySet();
		
		if (strings.isEmpty()) return null;
		
		
		String[] retorno = new String[strings.size()];
		Arrays.sort(strings.toArray(retorno));
		return retorno;
	}

	/**
	 * Obtem um vetor com os nomes dos tipos de jogadores de
	 * Poquer Fechado vlidos.
	 * @return Vetor com os nomes dos tipos disponveis.
	 */
	public String[] getJogadoresDePoquerFechado() {
		Set<String> strings = jogadoresFechado.keySet();
		
		if (strings.isEmpty()) return null;
		
		
		String[] retorno = new String[strings.size()];
		Arrays.sort(strings.toArray(retorno));
		return retorno;
	}
	
	/**
	 * Obtm um jogo de Texas Hold'em.
	 * @return O jogo criado.
	 * @throws ErroPoquer Um erro, caso no seja possvel criar o jogo.
	 */
	public JogoDeTexasHoldem getJogoTexasHoldem() throws ErroPoquer {
		if (jogoTexas != null) {
			try {
				return jogoTexas.newInstance();
			} catch (InstantiationException e) {
				throw new ErroPoquer("No foi possvel instanciar o jogo de texas hold'em.", e);
			} catch (IllegalAccessException e) {
				throw new ErroPoquer("O applet no tem permisso de instanciar o jogo de texas hold'em.", e);
			}
		} else {
			throw new ErroPoquer("No existe um jogo de texas hold'em disponvel.");
		}
	}
	
	/**
	 * Obtm um jogo de Pquer fechado.
	 * @return O jogo criado.
	 * @throws ErroPoquer Um erro, caso no seja possvel criar o jogo.
	 */
	public JogoDePoquerFechado getJogoPoquerFechado() throws ErroPoquer {
		if (jogoTexas != null) {
			try {
				return jogoFechado.newInstance();
			} catch (InstantiationException e) {
				throw new ErroPoquer("No foi possvel instanciar o jogo de poquer fechado.", e);
			} catch (IllegalAccessException e) {
				throw new ErroPoquer("O applet no tem permisso de instanciar o jogo de poquer fechado.", e);
			}
		} else {
			throw new ErroPoquer("No existe um jogo de poquer fechado disponvel.");
		}
	}
	
	/**
	 * Obtm um jogador de poquer fechado controlado pelo computador a ser
	 * usado pela interface.
	 * @param tipo O tipo do jogador.
	 * @return O jogador criado.
	 * @throws ErroPoquer Caso no seja possvel criar o jogador.
	 */
	public JogadorDePoquerFechadoIHC getComputadorDePoquerFechado(String tipo) throws ErroPoquer {
		Class<? extends JogadorDePoquerFechado> classe = jogadoresFechado.get(tipo);
		
		if (classe == null) throw new ErroPoquer("Jogador '" + tipo + "' indisponvel.");
		
		try {
			JogadorDePoquerFechado j = classe.newInstance();
			
			return new JogadorComputadorDePoquerFechado(this.interfaceHTML, j);
		} catch (InstantiationException e) {
			throw new ErroPoquer("No foi possvel criar o jogador do tipo '" + tipo + "'. Veja se h um construtor sem parmetros nessa classe."); 
		} catch (IllegalAccessException e) {
			throw new ErroPoquer("Sem acesso para criar o jogador do tipo '" + tipo + "'.");
		}
	}
	
	/**
	 * Obtm um jogador de poquer fechado controlado pelo usurio.
	 * @param nome O nome do usurio.
	 * @return O jogador criado.
	 */
	public JogadorDePoquerFechadoIHC getJogadorDePoquerFechado(String nome) {
		return new JogadorHumanoDePoquerFechado(nome, this.interfaceHTML);
	}
	
	/**
	 * Obtm um jogador de texas hold'em controlado pelo computador a ser
	 * usado pela interface.
	 * @param tipo O tipo do jogador.
	 * @return O jogador criado.
	 * @throws ErroPoquer Caso no seja possvel criar o jogador.
	 */
	public JogadorDeTexasHoldemIHC getComputadorDeTexasHoldem(String tipo) throws ErroPoquer {
		Class<? extends JogadorDeTexasHoldem> classe = jogadoresTexas.get(tipo);
		
		if (classe == null) throw new ErroPoquer("Jogador '" + tipo + "' indisponvel.");
		
		try {
			JogadorDeTexasHoldem j = classe.newInstance();
			
			return new JogadorComputadorDeTexasHoldem(this.interfaceHTML, j);
		} catch (InstantiationException e) {
			throw new ErroPoquer("No foi possvel criar o jogador do tipo '" + tipo + "'. Veja se h um construtor sem parmetros nessa classe."); 
		} catch (IllegalAccessException e) {
			throw new ErroPoquer("Sem acesso para criar o jogador do tipo '" + tipo + "'.");
		}
	}
	
	/**
	 * Obtm um jogador de texas hold'em controlado pelo usurio.
	 * @param nome O nome do usurio.
	 * @return O jogador criado.
	 */
	public JogadorDeTexasHoldemIHC getJogadorDeTexasHoldem(String nome) {
		return new JogadorHumanoDeTexasHoldem(nome, this.interfaceHTML);
	}
	
	/**
	 * Obtem um mapa com os jogadores de texas hold'em disponveis no applet.
	 * @return Um mapa com o nome e a classe dos jogadores de texas hold'em.
	 */
	private HashMap<String, Class<? extends JogadorDeTexasHoldem>> obterClassesJogadoresTexas(XMLStreamReader parser) throws ErroPoquer, XMLStreamException {
		HashMap<String, Class<? extends JogadorDeTexasHoldem>> classes = new HashMap<String, Class<? extends JogadorDeTexasHoldem>>();
		String nome, nomeClasse;
		Class<? extends JogadorDeTexasHoldem> classe = null;
		
		while(parser.hasNext()) {
			switch (parser.next()) {
			case XMLStreamConstants.START_ELEMENT:
				if (XML_JOGADOR.equalsIgnoreCase(parser.getLocalName())) {
					// obtendo o nome
					nome = null;
					for (int i = 0; i < parser.getAttributeCount() && nome == null; i++) {
						if (XML_NOME_JOGADOR.equalsIgnoreCase(parser.getAttributeName(i).getLocalPart()))
							nome = parser.getAttributeValue(i);
					}
					
					if (nome != null) {
						nomeClasse = parser.getElementText();
						System.out.println("[DEBUG] Jogador Texas: " + nome + " - " + nomeClasse);
						try {
							classe = Class.forName(nomeClasse).asSubclass(JogadorDeTexasHoldem.class);
							classes.put(nome, classe);
						} catch (ClassNotFoundException e) {
							// Classe no encontrada! No est no classpath!!!
							throw new ErroPoquer("Classe " + nomeClasse + " no disponvel.", e);
						} catch (ClassCastException e) {
							// Classe no  do tipo certo
							throw new ErroPoquer("Classe " + nomeClasse + " no  um Jogador de Texas Hold'em.", e);
						}
					}
				}
				break;
			case XMLStreamConstants.END_ELEMENT:
				if (XML_JOGADORES.equalsIgnoreCase(parser.getLocalName())) {
					// fim
					return classes;
				}
			}
		}
		
		return classes;
	}
	
	/**
	 * Obtem um mapa com os jogadores de poquer fechado disponveis no applet.
	 * @return Um mapa com o nome e a classe dos jogadores de poquer fechado.
	 */
	private HashMap<String, Class<? extends JogadorDePoquerFechado>> obterClassesJogadoresFechado(XMLStreamReader parser) throws ErroPoquer, XMLStreamException {
		HashMap<String, Class<? extends JogadorDePoquerFechado>> classes = new HashMap<String, Class<? extends JogadorDePoquerFechado>>();
		String nome, nomeClasse;
		Class<? extends JogadorDePoquerFechado> classe = null;
		
		while(parser.hasNext()) {
			switch (parser.next()) {
			case XMLStreamConstants.START_ELEMENT:
				if (XML_JOGADOR.equalsIgnoreCase(parser.getLocalName())) {
					// obtendo o nome
					nome = null;
					for (int i = 0; i < parser.getAttributeCount() && nome == null; i++) {
						if (XML_NOME_JOGADOR.equalsIgnoreCase(parser.getAttributeName(i).getLocalPart()))
							nome = parser.getAttributeValue(i);
					}
					
					if (nome != null) {
						nomeClasse = parser.getElementText();
						System.out.println("[DEBUG] Jogador Poquer Fechado: " + nome + " - " + nomeClasse);
						try {
							classe = Class.forName(nomeClasse).asSubclass(JogadorDePoquerFechado.class);
							classes.put(nome, classe);
						} catch (ClassNotFoundException e) {
							// Classe no encontrada! No est no classpath!!!
							throw new ErroPoquer("Classe " + nomeClasse + " no disponvel.", e);
						} catch (ClassCastException e) {
							// Classe no  do tipo certo
							throw new ErroPoquer("Classe " + nomeClasse + " no  um Jogador de Poquer Fechado.", e);
						}
					}
				}
				break;
			case XMLStreamConstants.END_ELEMENT:
				if (XML_JOGADORES.equalsIgnoreCase(parser.getLocalName())) {
					// fim
					return classes;
				}
			}
		}
		
		return classes;
	}
}
