package br.usp.poli.java.ihc.applet;

/*
Copyright (C) 2012 Fbio Levy Siqueira (levy.siqueira@gmail.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

import java.util.ArrayList;

import netscape.javascript.JSObject;
import br.usp.poli.java.Carta;
import br.usp.poli.java.ihc.ErroPoquer;
import br.usp.poli.java.ihc.JogadorIHC;

/**
 * Classe com os mtodos que acessam/so acessados pelo applet.<br />
 * Os mtodos acessveis pelo applet so os pblicos.
 * @author FLevy
 */
public class InterfaceHTML {
	static enum Posicao{SUPERIOR, INFERIOR};
	
	private final static int TEMPO_ESPERA_ACK = 100;
	
	private JogadorIHC computador, usuario;
	private JSObject janela;
	private boolean temRespostaAposta = false;
	private int aposta;
	private boolean temRespostaCobrir = false;
	private boolean cobrir;
	private boolean temRespostaTrocar = false;
	private CartaImpl[] trocar;
	private boolean ack = false;
	
	/**
	 * Cria uma interface HTML passando a janela do browser que ela ir acessar.
	 * @param janela A janela do browser.
	 */
	public InterfaceHTML(JSObject janela) {
		this.janela = janela;
	}
	
	/**
	 * Registra os jogadores para uma partida, alocando cada um em sua posio da mesa.
	 * @param computador O jogador representando o computador.
	 * @param usuario O jogador representando o usurio.
	 */
	public void registrarJogadores(JogadorIHC computador, JogadorIHC usuario) {
		this.computador = computador;
		this.usuario = usuario;
	}
	
	/**
	 * Avisa o depsito de fichas.
	 * @param posicao A posio que deposita fichas.
	 * @param deposito O nmero de fichas depositadas.
	 * @param fichasDisponiveis As fichas disponveis.
	 */
	void depositar(JogadorIHC jogador, int deposito, int fichasDisponiveis) {
		String[] enviar = {getPosicao(jogador), deposito+"", fichasDisponiveis+""};
		enviar("depositar", enviar);
	}
	
	/**
	 * Avisa as cartas recebidas.
	 * @param mao Um ArrayList contendo as cartas recebidas.
	 */
	void receberCartas(ArrayList<Carta> mao) {
		// O JavaScript no consegue receber um vetor direito, aparentemente
		String[] enviar = {""};
		for (int i = 0; i < mao.size(); i++) {
			enviar[0] += CartaImpl.toString(mao.get(i));
			
			// para dividir corretamente
			if (i != mao.size() - 1) enviar[0] += "|"; 
		}
		enviar("receberCartas", enviar);
	}
	
	/**
	 * Pede para o jogador apostar.
	 * @param fichasDisponiveis As fichas que o jogador tem disponveis.
	 * @param valorDaApostaMinima O valor da aposta mnima (para cobrir a aposta do adversrio).
	 * @param pote O pote at o momento.
	 */
	void pedirParaApostar(int fichasDisponiveis, int valorDaApostaMinima, int pote) {
		String[] enviar = {fichasDisponiveis + "", valorDaApostaMinima + "", pote +""};
		janela.call("pedirParaApostar", enviar);
	}
	
	/**
	 * Mostra a aposta feita pelo computador.
	 * @param aposta O valor da aposta feita pelo computador.
	 */
	void mostrarAposta(int aposta) {
		String[] enviar = {aposta + ""};
		enviar("mostrarAposta", enviar);
	}
	
	/**
	 * Mtodo chamado pela pgina para avisar que uma aposta foi feita.
	 * @param fichas O nmero de fichas da aposta.
	 */
	public void apostar(int fichas) {
		aposta = fichas;
		temRespostaAposta = true;
	}
	
	/**
	 * Informa se j h uma resposta em relao  aposta.<br>
	 * Este mtodo (assim como o "apostar" e "getAposta")  necessrio ao considerar
	 * que a comunicao com a pgina  assncrona.
	 * @return Se tem resposta ou no.
	 */
	boolean temRespostaAposta() {
		return temRespostaAposta;
	}
	
	/**
	 * Obtm a aposta feita.
	 * @return A aposta.
	 */
	int getAposta() {
		temRespostaAposta = false;
		return aposta;
	}
	
	/**
	 * Pergunta para o usurio se ele quer cobrir uma aposta. 
	 * @param fichasDisponiveis O nmero de fichas disponveis.
	 * @param fichasNecessariasParaCobrir O nmero de fichas necessrias para cobrir.
	 * @param pote O pote atual.
	 */
	void perguntarSeQuerCobrir(int fichasDisponiveis, int fichasNecessariasParaCobrir, int pote) {
		String[] enviar = {fichasDisponiveis + "", fichasNecessariasParaCobrir + "", pote +""};
		janela.call("perguntarSeQuerCobrir", enviar);
	}
	
	/**
	 * Mtodo chamado pela pgina para avisar se vai cobrir ou no.
	 * @param cobrir Se cobre a aposta ou no.
	 */
	public void cobrir(boolean cobrir) {
		this.cobrir = cobrir;
		temRespostaCobrir = true;
	}
	
	/**
	 * Informa se j h uma resposta em relao  cobrir.
	 * @return Se j tem resposta ou no.
	 */
	boolean temRespostaCobrir() {
		return temRespostaCobrir;
	}
	
	/**
	 * Obtm a resposta em relao ao cobrir.
	 * @return Se o usurio cobre a aposta (true) ou no (false).
	 */
	boolean getRespostaCobrir() {
		temRespostaCobrir = false;
		return cobrir;
	}
	
	/**
	 * Informa se o adversrio cobre ou no a aposta.
	 * @param cobrir Se ele cobre (true) ou no (false).
	 * @param fichasDisponiveis O nmero de fichas disponveis pelo adversrio.
	 * @param fichasNecessariasParaCobrir O nmero de fichas necessrias para o adversrio cobrir.
	 */
	void adversarioCobre(boolean cobrir, int fichasDisponiveis, int fichasNecessariasParaCobrir) {
		String[] enviar = {cobrir + "", fichasDisponiveis + "", fichasNecessariasParaCobrir + ""};
		enviar("cobrir", enviar);
	}
	
	/**
	 * Mostra a mo do adversrio.
	 * @param mao Um ArrayList contendo as cartas da mo.
	 */
	void mostrar(ArrayList<Carta> mao) {
		// O JavaScript no consegue receber um vetor direito, aparentemente
		String[] enviar = {""};
		for (int i = 0; i < mao.size(); i++) {
			enviar[0] += CartaImpl.toString(mao.get(i));
			
			// para dividir corretamente
			if (i != mao.size() - 1) enviar[0] += "|"; 
		}
		enviar("mostrar", enviar);
	}
	
	/**
	 * Informa se o jogador ganhou a mo.
	 * @param jogador O jogador.
	 * @param resultado O resultado (1 se ganhou, 0 se empatou e -1 se perdeu).
	 * @param pote O pote em jogo.
	 * @param fichasDisponiveis O nmero de fichas que o jogador tem disponvel.
	 */
	void avisarGanhouMao(JogadorIHC jogador, int resultado, int pote, int fichasDisponiveis) {
		String[] enviar = {getPosicao(jogador), resultado + "", fichasDisponiveis + "", pote +""};
		enviar("avisarGanhouMao", enviar);
	}
	
	/**
	 * Avisa o fim do jogo.
	 * @param usuarioGanhou Se o jogador usurio ganhou (true) ou no (false).
	 */
	void avisarFimDoJogo(boolean usuarioGanhou) {
		String[] enviar = {usuarioGanhou + ""};
		janela.call("avisarFimDoJogo", enviar);
	}
	
	/**
	 * Informa o flop.
	 * @param flop1 A primeira carta do flop.
	 * @param flop2 A segunda carta do flop.
	 * @param flop3 A terceira carta do flop.
	 */
	void avisarFlop(Carta flop1, Carta flop2, Carta flop3) {
		String[] cartas = {CartaImpl.toString(flop1), CartaImpl.toString(flop2), CartaImpl.toString(flop3)};
		enviar("flop", cartas);
	}
	
	/**
	 * Avisa o turn.
	 * @param turn A carta do turn.
	 */
	void avisarTurn(Carta turn) {
		String[] cartas = {CartaImpl.toString(turn)};
		enviar("turn", cartas);		
	}
	
	/**
	 * Avisa o river.
	 * @param river A carta do river.
	 */
	void avisarRiver(Carta river) {
		String[] cartas = {CartaImpl.toString(river)};
		enviar("river", cartas);
	}
	
	/**
	 * Pede para perguntar se quer trocar cartas.
	 */
	void perguntarTrocarCartas() {
		// No precisa de ACK
		janela.call("perguntarTrocarCartas", null);
	}
	
	/**
	 * Mtodo chamado pela pgina para avisar as cartas a serem trocadas.
	 * @param cartas As cartas a serem trocadas.
	 */
	public void trocarCartas(String cartas) {
		if (cartas == null || cartas.length() == 0) {
			trocar = null;
			temRespostaTrocar = true;
		} else {
			try {
				String[] nomes = cartas.split("\\|");
				ArrayList<CartaImpl> resposta = new ArrayList<CartaImpl>();
				
				for (String c : nomes) {
					if (!"".equals(c)) {
						System.out.println("Carta: " + c);
						resposta.add(new CartaImpl(c));
					}
				}
				
				trocar = new CartaImpl[resposta.size()];
				resposta.toArray(trocar);
				
				temRespostaTrocar = true;
			} catch (ErroPoquer e) {
				e.printStackTrace();
				// se mandar mensagem, gera erro e InterruptedException
				trocar = null;
				temRespostaTrocar = false;
			}
		}
	}
	
	/**
	 * Informa se tem resposta do "trocar".
	 * @return Se a pgina j respondeu ou no.
	 */
	boolean temRespostaTrocar() {
		return temRespostaTrocar;
	}
	
	/**
	 * Obtm as cartas a serem trocadas.
	 * @return Um vetor com as cartas.
	 */
	CartaImpl[] getCartasTrocar() {
		temRespostaTrocar = false;
		return trocar;
	}
	
	/**
	 * Informa as novas cartas do jogador
	 * @param jogador O jogador a trocar cartas.
	 * @param novas As novas cartas.
	 */
	void novasCartas(JogadorIHC jogador, Carta[] novas) {
		if (jogador == computador) {
			String[] enviar = new String[1];
			
			if (novas == null || novas.length == 0) {
				enviar[0] = "0";
			} else {
				enviar[0] = novas.length + "";
			}
			enviar("avisarComputadorTrocouCartas", enviar);
		} else {
			if (novas == null || novas.length == 0) return;
			
			// O JavaScript no consegue receber um vetor direito, aparentemente
			String[] enviar = {""};
			for (int i = 0; i < novas.length; i++) {
				enviar[0] += CartaImpl.toString(novas[i]);
				
				// para dividir corretamente
				if (i != novas.length - 1) enviar[0] += "|"; 
			}
			enviar("avisarNovasCartas", enviar);
		}
	}
	
	/**
	 * Avisa uma mensagem de erro.
	 * @param mensagem A mensagem de erro.
	 */
	void erro(String mensagem) {
		System.out.println("[DEBUG] Erro: " + mensagem);
		String[] enviar = {mensagem};
		janela.call("erro", enviar);
	}
	
	/**
	 * Envia uma mensagem para a pgina, esperando por um ACK.
	 * @param metodo O mtodo do javascript a ser chamado.
	 * @param parametros Um vetor com os parmetros a serem passados.
	 */
	private void enviar(String metodo, String[] parametros) {
		janela.call(metodo, parametros);
		while (!houveAck()) {
			try {
				Thread.sleep(TEMPO_ESPERA_ACK);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Mtodo que permite  interface web avisar que ela terminou
	 * de executar o que foi pedido. <br />
	 * Isso  necessrio pois as chamadas aos mtodos web so sncronas, mas
	 * nem sempre os detalhes grficos da interface podem ser feitos com
	 * um mtodo s (por exemplo, quando h temporizao para virar uma carta).
	 * Assim, cada mtodo tem que dar um ack, avisando que terminou.
	 */
	public void ack() {
		ack = true;
	}
	
	/**
	 * Informa se houve um ack.
	 * @return Se houve o ack ou no.
	 */
	private boolean houveAck() {
		if (ack) {
			ack = false;
			return true;
		}
		
		return false;
	}
	
	/**
	 * Obtm a String da posicao de um jogador.
	 * @param O jogador o qual se quer saber a posio.
	 * @return A String representando a posio do jogador,
	 * ou null se o jogador no foi registrado.
	 */
	private String getPosicao(JogadorIHC jogador) {
		if (jogador == computador) return Posicao.SUPERIOR.toString();
		else if (jogador == usuario) return Posicao.INFERIOR.toString();
		return null;
	}
}
