package br.usp.poli.java.ihc.applet;

/*
Copyright (C) 2012 Fbio Levy Siqueira (levy.siqueira@gmail.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

import java.util.ArrayList;
import java.util.Arrays;

import br.usp.poli.java.Carta;
import br.usp.poli.java.Jogador;
import br.usp.poli.java.ihc.JogoInterrompido;

/**
 * Classe que encapsula um jogador controlado pelo computador e delega para ele
 * as chamadas de mtodo.
 * @author FLevy
 */
abstract class JogadorComputador extends JogadorIHCImpl {
	protected Jogador jogador;
	private final static String NOME = "Computador";
	
	public JogadorComputador(InterfaceHTML interfaceHTML, Jogador interno) {
		super(NOME, interfaceHTML);
		this.jogador = interno;
	}
	
	public void avisarDeposito(int deposito, int fichasDisponiveis) {
		if (interromper) throw new JogoInterrompido();
		
		this.jogador.avisarDeposito(deposito, fichasDisponiveis);
		interfaceHTML.depositar(this, deposito, fichasDisponiveis);
	}

	public void receberCartas(Carta[] mao) {
		if (interromper) throw new JogoInterrompido();
		
		this.mao = new ArrayList<Carta>(Arrays.asList(mao));
		this.jogador.receberCartas(mao);
		// No avisa a interface. O outro jogador faz isso.
	}

	public int apostar(int fichasDisponiveis, int valorDaApostaMinima, int pote) {
		if (interromper) throw new JogoInterrompido();
		
		int aposta = jogador.apostar(fichasDisponiveis, valorDaApostaMinima, pote);
		interfaceHTML.mostrarAposta(aposta);
		
		return aposta;
	}

	public boolean cobrir(int fichasDisponiveis,
			int fichasNecessariasParaCobrir, int pote) {
		if (interromper) throw new JogoInterrompido();
		
		boolean cobre = jogador.cobrir(fichasDisponiveis, fichasNecessariasParaCobrir, pote);
		interfaceHTML.adversarioCobre(cobre, fichasDisponiveis, fichasNecessariasParaCobrir);
		
		return cobre;
	}

	public Carta[] mostrar() {
		if (interromper) throw new JogoInterrompido();
		interfaceHTML.mostrar(this.mao);
		
		return jogador.mostrar();
	}

	public void avisarGanhouMao(int resultado, int pote, int fichasDisponiveis) {
		if (interromper) throw new JogoInterrompido();
	
		interfaceHTML.avisarGanhouMao(this, resultado, pote, fichasDisponiveis);
		jogador.avisarGanhouMao(resultado, pote, fichasDisponiveis);
	}
	
	public void avisarFimDoJogo(boolean vencedor) {
	}

}
