package br.usp.poli.java.ihc.applet;

/*
Copyright (C) 2012 Fbio Levy Siqueira (levy.siqueira@gmail.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

import java.util.ArrayList;
import java.util.Arrays;

import br.usp.poli.java.Carta;
import br.usp.poli.java.ihc.JogoInterrompido;

/**
 * Representa um Jogador controlado pelo usurio no jogo de poquer,
 * usando a interface HTML.
 * @see InterfaceHTML
 * @author FLevy
 */
abstract class JogadorHumano extends JogadorIHCImpl {
	public JogadorHumano(String nome, InterfaceHTML interfaceHTML) {
		super(nome, interfaceHTML);
	}
	
	public void avisarDeposito(int deposito, int fichasDisponiveis) {
		if (interromper) throw new JogoInterrompido();
		interfaceHTML.depositar(this, deposito, fichasDisponiveis);
	}

	public void receberCartas(Carta[] mao) {
		if (interromper) throw new JogoInterrompido();
		this.mao = new ArrayList<Carta>(Arrays.asList(mao));
		interfaceHTML.receberCartas(this.mao);
	}

	public int apostar(int fichasDisponiveis, int valorDaApostaMinima, int pote) {
		if (interromper) throw new JogoInterrompido();
		interfaceHTML.pedirParaApostar(fichasDisponiveis, valorDaApostaMinima, pote);
		
		// Esperando pela aposta
		while (!interfaceHTML.temRespostaAposta() && !interromper) {
			try {
				Thread.sleep(TEMPO_ESPERA);
			} catch (InterruptedException e) {
				interfaceHTML.erro("Erro ao obter o valor da aposta.");
				throw new JogoInterrompido();
			}
		}
		
		if (interromper) throw new JogoInterrompido();
		
		return interfaceHTML.getAposta();
	}

	public boolean cobrir(int fichasDisponiveis,
			int fichasNecessariasParaCobrir, int pote) {
		if (interromper) throw new JogoInterrompido();
		interfaceHTML.perguntarSeQuerCobrir(fichasDisponiveis, fichasNecessariasParaCobrir, pote);
		
		// Esperando pela aposta
		while (!interfaceHTML.temRespostaCobrir() && !interromper) {
			try {
				Thread.sleep(TEMPO_ESPERA);
			} catch (InterruptedException e) {
				interfaceHTML.erro("Erro ao obter o valor da aposta.");
				throw new JogoInterrompido();
			}
		}
		
		if (interromper) throw new JogoInterrompido();
		
		return interfaceHTML.getRespostaCobrir();
	}

	public Carta[] mostrar() {
		if (interromper) throw new JogoInterrompido();
		
		Carta[] cartas = new Carta[mao.size()];
		return mao.toArray(cartas);
	}

	public void avisarGanhouMao(int resultado, int pote, int fichasDisponiveis) {
		if (interromper) throw new JogoInterrompido();
		
		interfaceHTML.avisarGanhouMao(this, resultado, pote, fichasDisponiveis);
	}
	
	public void avisarFimDoJogo(boolean vencedor) {
		if (interromper) throw new JogoInterrompido();
		
		interfaceHTML.avisarFimDoJogo(vencedor);
	}
}
