package br.usp.poli.java.ihc.applet;

/*
Copyright (C) 2012 Fbio Levy Siqueira (levy.siqueira@gmail.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

import java.util.ArrayList;
import java.util.Arrays;

import br.usp.poli.java.Carta;
import br.usp.poli.java.ihc.JogadorDePoquerFechadoIHC;
import br.usp.poli.java.ihc.JogoInterrompido;

/**
 * Jogador de poquer fechado controlado pelo usurio.
 * @author FLevy
 */
public class JogadorHumanoDePoquerFechado extends JogadorHumano implements JogadorDePoquerFechadoIHC {

	public JogadorHumanoDePoquerFechado(String nome,
			InterfaceHTML interfaceHTML) {
		super(nome, interfaceHTML);
	}

	public Carta[] trocarCartas(int numeroDeCartasTrocadasAdversario) {
		if (interromper) throw new JogoInterrompido();
		
		interfaceHTML.perguntarTrocarCartas();
		
		// Esperando pela resposta
		while (!interfaceHTML.temRespostaTrocar() && !interromper) {
			try {
				Thread.sleep(TEMPO_ESPERA);
			} catch (InterruptedException e) {
				e.printStackTrace();
				interfaceHTML.erro("Erro ao perguntar sobre a troca de cartas.");
				throw new JogoInterrompido();
			}
		}
		
		if (interromper) throw new JogoInterrompido();
		
		CartaImpl[] cartas = interfaceHTML.getCartasTrocar();
		
		if (cartas == null || cartas.length == 0) return null;
		
		ArrayList<Carta> trocar = new ArrayList<Carta>();
		for (CartaImpl c : cartas) {
			for (int i = 0; i < mao.size(); i++) {
				if (c.compareTo(mao.get(i)) == 0) {
					trocar.add(mao.remove(i));
				}
			}
		}
		
		Carta[] retorno = new Carta[trocar.size()];
		trocar.toArray(retorno);
		
		return retorno;
		
	}

	public void receberNovasCartas(Carta[] cartas) {
		if (interromper) throw new JogoInterrompido();
		
		if (cartas == null || cartas.length == 0) return;
		
		this.mao.addAll(Arrays.asList(cartas));
		
		interfaceHTML.novasCartas(this, cartas);
	}

}
